/*--------------------------------------------------------------------
 *    The GMTPLUS-system:	@(#)gmtplusinfo.c	1.2	9/12/95
 *
 *  Based on...
 *    The GMT-system:	@(#)gmtinfo.c	2.4  7/22/93
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * gmtinfo prints out a string of information pertaining to each specified leg.
 * The parameters are: legname, year, no of records in file, no of gravity,
 * magnetics, and bathymetry points, area covered in lat/lon, and the d/m/yr
 * of the first and last records in the file(s).
 *
 * Author:	Paul Wessel
 * Date:	3-FEB-1988
 *
 *
 * GMTPlus version 03/10/95 jcm
 * Modifications made for gmtplus database system.
 *
 */
#include <gmt.h>
#include "gmtplus_mgg.h"

#define NODATA (-32000)

struct GMT *gmt;

int
main (int argc, char *argv[])
{
	int leg_year, n_records, rec, n_grv, n_mag, n_top;
	int begin_day, begin_mo, begin_yr, end_day, end_mo, end_yr;
	int hour, minute, second, nfiles=0, i, j, verbose = FALSE;
	int n_data[14];
	double xmin1, xmax1, xmin2, xmax2, ymin, ymax, xmin, xmax, lat, lon, old_x;
	char agency[10] = "", cruise[80] = "", greenwich, gmtfile[100], gmtheader[100];
	FILE *fp, *hp;
	struct GMT_PLUS_REC record;
	struct COL_N col;
	
	gmtpath_init();
	read_gmt_master_index();
	
	for (i = 1; i < argc; i++) 
		if ((!strcmp(argv[i], "-v")) || (!strcmp(argv[i], "-V"))) verbose = TRUE;
	
	for (i = 1; i < argc; i++) {	/* Loop over gmt files */
		if (argv[i][0] == '-') continue;
  		if (gmtpath_func (gmtfile, gmtheader, argv[i])) {
   			fprintf(stderr,"gmtplusinfo : Cannot find leg %s\n", argv[i]);
     			continue;
  		}
		if ((hp = fopen (gmtheader, "r")) == NULL) {
			fprintf(stderr,"gmtplusinfo: Could not open header file %s\n", gmtheader);
			continue;
		}
		if ((fp = fopen (gmtfile, "r")) == NULL) {
			fprintf(stderr,"gmtplusinfo: Could not open data file %s\n", gmtfile);
			fclose(hp);
			continue;
		}
		nfiles++;
	
		if (verbose) fprintf (stderr, "Now reading leg %s\n", argv[i]);
		
		/* Read header file containing start-year, n_records and info */
	
		read_gmt_header(hp, &col, &leg_year, &n_records, agency, cruise);
		read_first_gmt_datarec(fp, &col);
	
		gmt = gmt_init (leg_year);	/* Initialize gmt_structure */
	
		/* Start reading data from file */
	
		xmin1 = xmin2 = 360.0;
		xmax1 = xmax2 = -360.0;
		ymin = 180.0;
		ymax = -180.0;
		for (j=0;j<14;j++) n_data[j] = 0;
		greenwich = FALSE;
		for (rec = 0; rec < n_records; rec++) {
			/*if (fread((char *)(&record), 40, 1, fp) != 1) {*/
			/* 08/31/95 jcm Read to block out bad flagged data */
			if (read_data_record(&record, &col, fp) == -1) {
				fprintf(stderr,"gmtplusinfo: Error reading data record no %d\n",rec);
				exit(-1);
			}
			if (rec == 0)
				gmt_date(record.ldata[col.tsec],&begin_yr,&begin_mo,&begin_day,&hour,&minute,&second,gmt);

			for (j=0;j<6;j++)
				if (record.ldata[j] != NODATA) n_data[j]++;
			for (j=0;j<8;j++)
				if (record.sdata[j] != NODATA) n_data[j+6]++;
			lon = record.ldata[col.lon] * 0.000001;
			lat = record.ldata[col.lat] * 0.000001;
			if ((lon < 0) || (lon > 360) || (lat < -90) || (lat > 90)) {
				fprintf(stderr,"Bad lat/lon: %.5lf, %.5lf Record: %d\n",
					lat,lon,rec);
			} else {
				if (lon < 180.0) {
					xmin1 = MIN (lon, xmin1);
					xmax1 = MAX (lon, xmax1);
				}
				else {
					xmin2 = MIN (lon, xmin2);
					xmax2 = MAX (lon, xmax2);
				}
				ymin = MIN (lat, ymin);
				ymax = MAX (lat, ymax);
				if (rec > 0 && (fabs(old_x-lon) > 180.0))
					greenwich = TRUE;
				old_x = lon;
			}
		}
		fclose (fp);
		
		gmt_date (record.ldata[col.tsec],&end_yr,&end_mo,&end_day,&hour,&minute,&second,gmt);
		free ((char *)gmt);

		if (greenwich) {
			xmin = MAX(xmin1,xmin2);
			xmax = MIN(xmax1,xmax2);
		}
		else {
			xmin = MIN(xmin1,xmin2);
			xmax = MAX(xmax1,xmax2);
		}
		if (xmin > xmax) xmin -= 360.0;
		printf("Info: %s Year: %d N_recs: %d W: %.5lf E: %.5lf S: %.5lf N: %.5lf Begins: %d %d %d Ends: %d %d %d",
			cruise, leg_year, n_records, xmin, xmax, ymin, ymax,
			begin_day, begin_mo, begin_yr, end_day, end_mo, end_yr);
		for (j=0; j<6; j++) {
			if ((n_data[j] != 0) && (col.tsec != j)
				&& (col.lat != j) && (col.lon != j)
				&& (col.bad_flag != j)
				&& (first_record.ldata[j] != -1)) {
				printf(" %s: %d",gmt_column_name[first_record.ldata[j]], 
						n_data[j]);
			}
		}
		for (j=0; j<8; j++) {
			if ((n_data[j+6] != 0) && (col.tsec != j+6)
				&& (col.lat != j+6) && (col.lon != j+6)
				&& (col.bad_flag != j+6)
				&& (first_record.sdata[j] != -1)) {
				printf(" %s: %d",gmt_column_name[first_record.sdata[j]], 
						n_data[j+6]);
			}
		}
		printf("\n");

	}
	
	if (nfiles == 0) {
		fprintf(stderr, "usage: gmtplusinfo leg(s)\n");
		exit(-1);
	}
	
	exit (0);
}
