/*--------------------------------------------------------------------
 *    The GMTPLUS-system:	@(#)uwmtogmtplus.c	1.2	9/12/95
 *
 *  Based on...
 *    The GMT-system:	@(#)mgd77togmt.c	2.6  7/23/93
 *
 *--------------------------------------------------------------------*/
/*
 * uwmtogmtplus reads a SIO uwmerge format data file and creates
 * the standard gmtplus file {*.gmtd, *.gmth}. If no time is provided,
 * a fake time is issued based on a year and time incerment.  Use
 * 1900 for these cruises and estimate the time_inc based on the
 * distance between 2 nav points and an assumed velocity.
 *
 * This program converts data only, and does not do any additional
 * calculations (suce as corrected meters or mag anomaly).
 *
 * GMTPlus 03/10/95 jcm
 *
 */
#include <unistd.h>
#include <time.h>
#include <gmt.h>
#include "gmtplus_mgg.h"
#if 0
/* Provided by the os from time.h */
extern long int timezone;  /* FIX: scary.  Where is this coming from? -kds */
#endif

#define NODATA -32000

struct GMT *gmt;
struct GMT_PLUS_REC *record;
struct COL_N col;
int T_INC = 60, fake_running_time = 0;
struct READ_RECORD {
       char argh[8];
       int thing1;
       short int thing2;
       short int thing3; 
       short int thing4;
       short int thing5; 
       int thing6;
       int thing7;
       int thing8;
       int thing9;
       int thing10;
};


/* FIX: I do not believe this works.  -kurt */
int decode_record (struct READ_RECORD *a_record, int tflag, int year_start,
                   int *time, int *lon, int*lat, int *graw, short *faa, short *mag, short *manom, short *itwt)
/*
struct READ_RECORD *a_record;
int tflag;
int year_start;
int *time, *lon, *lat, *graw;
short int *faa, *mag, *manom, *itwt;
*/
{
	int year, month, day, hour, min, sec, l_mag, l_top, test, bin, zone, l_twt;
    double lat1,lat2,lon1,lon2;
    int seconds;

    if (tflag) {    /* no time provided, use fake time */
       *time = fake_running_time;
       fake_running_time += T_INC;
    } else {
       *time = a_record->thing1 - year_start;
    }
	
	/* Get lat lon */
    lat1 = (int)(a_record->thing2 / 60);     /* degrees */
    lat2 = (float)a_record->thing3/10000.0    /* minutes */
            + (a_record->thing2 - (lat1*60));
    *lat = (lat1+(lat2/60.0))*1000000.0 - 90000000.0;
	
    lon1 = (int)(a_record->thing4 / 60);     /* degrees */
    lon2 = (float)a_record->thing5/10000.0    /* minutes */
            + (a_record->thing4 - (lon1*60));
    *lon = (lon1+(lon2/60.0))*1000000.0;
	
	
	/* Get gravity */
	
	if (a_record->thing10 == -999999)
		*faa = NODATA;
	else 
        /* source is mgal*100; dest is .1mgal*10; need (/100)(*10) */
		*faa = (short)((float)a_record->thing10/10.0);
		
	if (a_record->thing9 == -999999)
		*graw = NODATA;
    else 
        /* source is mgal*100; dest is .1mgal*10; need (/100)(*10) */
        *graw = (int)((float)a_record->thing9/10.0);
		
	/* Get magnetics */

    if (a_record->thing8 == -999999)
       *manom = NODATA;
    else
       *manom = (short)((float)a_record->thing8/10.0);
	

    if (a_record->thing7 == -999999)
       *mag = NODATA;
    else
       *mag = (short)(((float)a_record->thing7/10.0) - 50000);

	
	/* Get Bathymetry */
    
    if (a_record->thing6 == -999999)
       *itwt = NODATA;
    else 
       /* source is sec*10000, want ms (sec*1000) thus / 10.0 */
       *itwt = (short)(a_record->thing6/10.0);
  
    /* note: no conversion to depth; look at mgd77togmtplus.c later */

	return (0);
}



int
main (int argc, char *argv[])
{
  int i, rec, n_records, n_alloc = GMT_CHUNK, leg_year = 0;
  int t_flag = FALSE, error = FALSE, header = FALSE;
  char file[80], agency[10] = "", legfname[80], line[512];
  char tempfile[80], dummy, dummy2[10];
       FILE *fp = NULL, *fpo, *hpo, *tpo;
    int count;
    int seconds,adepth,amtot,amanom,agraw,agfaa;
    short int alat,alat_dec,alon,alon_dec;
    struct READ_RECORD a_record; 
    time_t year_start, year_gmt;
    struct tm time_start, *gmt_start;
    struct tm my_time;

    strcpy(tempfile,"/tmp/TEMPXXXXXX");	
    mktemp(tempfile);
    tpo = fopen(tempfile, "w");
	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
				case 'Y':
					leg_year = atoi (&argv[i][2]);
					break;
				case 'I':	/* No time in data file, specify time increment between points */
					T_INC = atoi (&argv[i][2]);
					t_flag = TRUE;	/* fake time starts at 1/1/1900 @ 00:00:00 hr */
					break;
				case 'F':
					strcpy (legfname, &argv[i][2]);
					break;
				case 'A':
					strcpy (agency, &argv[i][2]);
					break;
				default:
					error = TRUE;
					break;
			}
		}
		else
			fp = fopen (argv[i], "r");
	}
	
	if (fp == NULL) fp = stdin;
	
	if (argc == 1 || error) {
		fprintf (stderr, "usage: uwmtogmtplus -Ffilename [-Yleg_year] [-A<10 char agency name>] [SIOuwmrg-file -Itime_increment]\n");
		fprintf (stderr, "	-Y sets start year\n");
		fprintf (stderr, "	-I sets fake timeincrement for files without time information\n");
		fprintf (stderr, "	-F sets gmtfilename prefix (e.g., without path or .gmt)\n");
		fprintf(stderr,"	-A sets optional 10 char info for gmt header.\n");
		
		exit (-1);
	}

	read_gmt_master_index();
	col.tsec = 0;
	col.lat = 1;
	col.lon = 2;
	col.graw = 3;
	col.gfaa = 6;
	col.twtt = 7;
	col.tcor = -1;
	col.mtot = 9;
	col.manm = 10;
	col.mflt = -1;
	col.bad_flag = -1;
	col.user_long[0] = -1;
	col.user_long[1] = -1;
	col.user_long[2] = -1;
	col.user_long[3] = -1;
	col.user_long[4] = -1;
	col.user_long[5] = -1;
	col.user_short[0] = -1;
	col.user_short[1] = -1;
	col.user_short[2] = -1;
	col.user_short[3] = -1;
	col.user_short[4] = -1;
	col.user_short[5] = -1;
	col.user_short[6] = -1;
	col.user_short[7] = -1;


    year_start = 0;
    if (leg_year != 0) {	
	   gmt = gmt_init (leg_year);
       time_start.tm_sec = 0;
       time_start.tm_min = 0;
       time_start.tm_hour = 0;
       time_start.tm_mday = 1;
       time_start.tm_mon = 0;
       if (leg_year > 1900) 
          time_start.tm_year = leg_year - 1900;
       else
          time_start.tm_year = leg_year;
       year_start = mktime(&time_start);

       /* FIXME: jjb hacking this in; do we need to call tzset, or has it already been done? */
       tzset();

/* since of old school gmtoff (east is > 0), and new timezone (west <0) has to be considered. */
/* year_start += time_start.tm_gmtoff; */

       year_start -= timezone;
    }


	record = (struct GMT_PLUS_REC *) GMT_memory ((char *)NULL, n_alloc, sizeof (struct GMT_PLUS_REC), "uwmtogmtplus");

	rec = 0;
    /* read any pedigree records */
    while ((dummy = fgetc(fp)) == '#') {
       ungetc(dummy, fp);
       fread(line, 40, sizeof(char), fp);	
       line[40] = '\0';
       fprintf(tpo,"%s\n",line);
    }
    ungetc(dummy, fp);
	while (fread ((char *)&a_record, 40, sizeof(char), fp)) {
        if ((leg_year == 0) && !t_flag) {	/* first data record, set time */
	   gmt_start = gmtime((time_t *)&(a_record.thing1));
           /* convert to jan1 of that year */
           leg_year = gmt_start->tm_year;
	       gmt = gmt_init (leg_year);
           gmt_start->tm_sec = 0;
           gmt_start->tm_min = 0;
           gmt_start->tm_hour = 0;
           gmt_start->tm_mday = 1;
           gmt_start->tm_mon = 0;
           year_start = mktime(gmt_start);
	   /* jjb hacking this in; do we need to call tzset, or has it already been done? */
tzset();
/* since of old school gmtoff (east is > 0), and new timezone (west <0) has to be considered. */
/* year_start += time_start.tm_gmtoff; */
 year_start -= timezone;
        }
		for (i=0;i<6;i++)
			record[rec].ldata[i] = 0;
		for (i=0;i<8;i++)
			record[rec].sdata[i] = 0;
		if (!decode_record (&a_record, t_flag, year_start,
			&record[rec].ldata[col.tsec],
			&record[rec].ldata[col.lon],
			&record[rec].ldata[col.lat],
			&record[rec].ldata[col.graw],
			&record[rec].sdata[col.gfaa-6],
			&record[rec].sdata[col.mtot-6],
			&record[rec].sdata[col.manm-6],
			&record[rec].sdata[col.twtt-6])) rec++;
		if (rec == n_alloc) {
			n_alloc += GMT_CHUNK;
			record = (struct GMT_PLUS_REC *) GMT_memory ((char *)record, n_alloc, sizeof (struct GMT_PLUS_REC), "uwmtogmtplus");
		}
	}
	if (fp != stdin) fclose (fp);
    fclose(tpo);
	free ((char *)gmt);
	n_records = rec;
	
	sprintf (file, "%s.gmth\0", legfname);
	if ((hpo = fopen (file, "w")) == NULL) {
		fprintf (stderr, "Could not create header file %s\n", file);
		exit (-1);
	}
	sprintf (file, "%s.gmtd\0", legfname);
	if ((fpo = fopen (file, "w")) == NULL) {
		fprintf (stderr, "Could not create data file %s\n", file);
		exit (-1);
	}

	write_gmt_header(hpo, &col, leg_year, n_records, agency, legfname, file);
    tpo = fopen(tempfile,"r");
	while (fgets (line, 512, tpo)) 
       fprintf(hpo,"%s",line);
    fclose(tpo);
	write_first_gmt_datarec(fpo, &col);

	for (rec = 0; rec < n_records; rec++) {
		if (fwrite((char *)(&record[rec]), 40, 1, fpo) != 1) {
			fprintf(stderr,"uwmtogmtplus: Error writing data record no %d\n",rec);
			exit(-1);
		}
	}

	fclose(fpo);
	fclose(hpo);
	
	free ((char *)record);
	
	exit(0);
}


